import os

from oscars_mgmnt_clms.parser.base import BaseParser
import re
from datetime import datetime as dt
import time
from abc import ABC, abstractmethod


class FilenameParser(BaseParser, ABC):

    def convert(self, product_file_info):
        values = {}
        # common properties for filename parsers
        fn = os.path.abspath(product_file_info.get_product_path())
        # filename without extension
        file_name = os.path.splitext(os.path.basename(fn))[0]
        file_name_parts = file_name.split("_")

        nominaldatetime = parse_fname_date(file_name_parts[3])
        values["nominaldatetime"] = nominaldatetime.isoformat() + "Z"

        # collection identifier is constructed from the collection-folder
        path_elem = os.path.dirname(fn).split(os.path.sep)

        # assuming files are stored ...
        nominal_year_str = str(nominaldatetime.year)
        #  ...  either in 3rd level below collection, ie collection/yyyy/yyyymmdd/file
        if path_elem[-2] == nominal_year_str and path_elem[-1].startswith(
                nominal_year_str
        ):
            collection_dir = path_elem[-3]
        else:
            #  ...  or in lowest folder level (no organization per date, e.g. static masks)
            collection_dir = path_elem[-1]

        parent_identifier = "_".join(["clms_global", collection_dir, self.filetype])
        values["parentidentifier"] = parent_identifier

        values["creationtime"] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values["catingestiondate"] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values["updatetime"] = time.strftime(
            "%Y-%m-%dT%H:%M:%SZ", product_file_info.storage.get_time_modified(fn)
        )

        # specific treatment
        specific_infos = self.splitted_treatment(product_file_info, values)

        # Generation of the output
        results = {}

        if self.jinja_template is None:
            self.with_template(self.template)

        for specific in specific_infos:
            result = self.jinja_template.render(
                dict=specific.values,
                dirpath=specific.product_file_info.get_product_path().rpartition("/")[
                    0
                ],
                products=specific.product_file_info.datas,
                previews=specific.product_file_info.previews,
                metadatas=specific.product_file_info.metadatas,
                related=specific.product_file_info.related,
                platforms=specific.values['platforms'],
            )
            results[specific.uid] = result
        return results

    @property
    @abstractmethod
    def filetype(self) -> str:
        raise NotImplementedError


def parse_fname_date(fname_date: str) -> dt:
    """
    Parse the date from the filename of a product.

    Nominal dates in filenames can take one of three forms:
        - YYYYMMDDhhmm (12 digits): the most common case
        - YEAR1----YEAR2 (4 digits, 4 hyphens, 4 digits) -> nominal date = Jan 1 of YEAR1:
            e.g. SWI static
        - YEAR1-YEAR2-MMDD (4 digits, hyphen, 4 digits, hyphen, 4 digits) -> nominal date = YEAR1/MM/DD:
            e.g. NDVI statistics
    """
    import re

    if re.search("[0-9]{12}", fname_date):
        return dt.strptime(fname_date, "%Y%m%d%H%M")
    elif re.search("[0-9]{4}----[0-9]{4}", fname_date):
        return dt.strptime(fname_date[0:4], "%Y")
    else:
        return dt.strptime(fname_date[0:4] + fname_date[-4:], "%Y%m%d")
